#![allow(bad_style)]
#![allow(non_snake_case)]
#![allow(non_upper_case_globals)]
#![allow(non_camel_case_types)]
#![allow(dead_code)]
#![allow(improper_ctypes)]
// Suppress clippy warnings for generated bindgen code
#![allow(clippy::missing_safety_doc)]
#![allow(clippy::ptr_offset_with_cast)]
#![allow(clippy::useless_transmute)]
#![no_std]

include!(concat!(env!("OUT_DIR"), "/bindings.rs"));
